----------------------------------------------------------------------------------
-- 
-- Date:    26.03.2013
-- Author:  Lucie Matusova, xmatus21
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity INTERFACE_ENT is
  generic (
    DATA_WIDTH : natural := 8;
	 REG_IN	   : natural := 7; -- number of senzoric signals    
	 ADDR_WIDTH : natural := 4  -- 3 would be enough (7 addresses)
  );
  port(
    CLK  : in std_logic;
    RST  : in std_logic;

    DATA : in std_logic_vector(DATA_WIDTH-1 downto 0);
    SEL  : in std_logic_vector(ADDR_WIDTH-1 downto 0);
    VLD  : in std_logic;

    DA   : out std_logic_vector(DATA_WIDTH-1 downto 0);
    DB   : out std_logic_vector(DATA_WIDTH-1 downto 0);
    DC   : out std_logic_vector(DATA_WIDTH-1 downto 0);
    DL   : out std_logic_vector(DATA_WIDTH-1 downto 0);
    DR   : out std_logic_vector(DATA_WIDTH-1 downto 0);
    DU   : out std_logic_vector(DATA_WIDTH-1 downto 0);
    DD   : out std_logic_vector(DATA_WIDTH-1 downto 0);
    RDY  : out std_logic
  );
end INTERFACE_ENT;

architecture INTERFACE_ARCH of INTERFACE_ENT is
  type arr_reg_in  is array(0 to REG_IN-1)  of std_logic_vector(DATA_WIDTH-1 downto 0);

  signal sel_delay : std_logic_vector(ADDR_WIDTH-1 downto 0);
  signal sel_del   : std_logic_vector(ADDR_WIDTH-1 downto 0);
  signal enable    : std_logic_vector( REG_IN-1 downto 0);
  signal we        : std_logic_vector( REG_IN-1 downto 0);  -- '1' when enable and data_valid
  signal arr_in    : arr_reg_in;
  signal data_valid: std_logic;
 
  component CHANGE_ENT is
      generic (
         DATA_WIDTH : integer := 4      
      );
   
      port (
         CLK     : in std_logic;
         RST     : in std_logic;
         DATA_IN : in std_logic_vector(DATA_WIDTH-1 downto 0);
         OUTPUT  : out std_logic
      );   
  end component;
 
  component registr_en
      generic ( 
         n : natural 
      );  
      port (
         CLK : in  std_logic;
         RST : in  std_logic;
         EN  : in  std_logic;
         I   : in  std_logic_vector(n-1 downto 0);
         O   : out std_logic_vector(n-1 downto 0) 
      );
  end component;
  
  component registr
      generic ( 
         n : natural 
      );  
      port (
         CLK : in  std_logic;
         RST : in  std_logic;
         I   : in  std_logic_vector(n-1 downto 0);
         O   : out std_logic_vector(n-1 downto 0) 
      );
  end component;  
  
  component decoder
      generic (
         ADDR_WIDTH : natural;
         EN_WIDTH   : natural
      );
      port (
         INPUT : in  std_logic_vector(ADDR_WIDTH-1 downto 0);
         EN    : out std_logic_vector(EN_WIDTH-1 downto 0)  
      );
  end component;

  -- ------------------------------------------------------------
  -- ------------------------------------------------------------
  
  begin

   --change_det : CHANGE_ENT
   --    generic map (
   --       DATA_WIDTH => 4
   --    )
   --    port map (
   --       CLK => CLK,
   --       RST => RST,
   --       DATA_IN => SEL,
   --       OUTPUT => data_valid
   --    );
  
  data_valid <= VLD;
      
  -- address decoder
	dec : decoder
      generic map(
         ADDR_WIDTH => ADDR_WIDTH,
         EN_WIDTH => REG_IN
      )
      port map(
         INPUT  => SEL,
         EN => enable
      );  

	--	registers for senzoric values
	r_in: for i in 0 to (REG_IN-1) generate
	  -- '1' when proper address comes and data are valid at this address
	  we(i) <= '1' when (enable(i) = '1' AND data_valid = '1') else '0';
	  
		r_in_i: registr_en generic map (n => DATA_WIDTH)
			port map (CLK => CLK, RST => RST, EN => we(i), I => DATA, O => arr_in(i));
	end generate r_in;

	-- delaying address signal so we can use it for RDY signal
    r_sel_del: registr generic map (n => ADDR_WIDTH)
		port map (CLK => CLK, RST => RST, I => SEL, O => sel_del);

	-- delaying address signal so we can use it for RDY signal
    r_sel_delay: registr generic map (n => ADDR_WIDTH)
		port map (CLK => CLK, RST => RST, I => sel_del, O => sel_delay);

	-- presuming that senzoric values are coming one by one 
	--  as we send it; that is A,B,C,L,R,U,D
	--  the addresses are also changing in this manner
	--  0...6 - addr(A)...addr(D)
	-- if there is an addr(D) on (delayed) SEL signal, all values
	--  should be accurate and we can read them out; RDY is '1'
	RDY <= '1' when (sel_delay = "0110" AND data_valid = '1') else '0';
	  
	DA <= arr_in(0);
	DB <= arr_in(1);
	DC <= arr_in(2);
	DL <= arr_in(3);
	DR <= arr_in(4);
	DU <= arr_in(5);
	DD <= arr_in(6);
  
end INTERFACE_ARCH;
