-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    bdu.vhd
-- Description:  Barrier Detection Unit
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         20.1.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity BDU_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16   
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic;
      ACT            : in  std_logic;

      -- ----------------- INPUT INTERFACE ----------------------------------
      S_0            : in  std_logic_vector(DATA_WIDTH-1 downto 0); 
      S_1            : in  std_logic_vector(DATA_WIDTH-1 downto 0);
      S_2            : in  std_logic_vector(DATA_WIDTH-1 downto 0);
      S_3            : in  std_logic_vector(DATA_WIDTH-1 downto 0);      

      -- ----------------- OUTPUT INTERFACE ---------------------------------      
      OUTPUT         : out std_logic_vector(DATA_WIDTH-1 downto 0);--4-bits vector (4-neighbourhood)
      VLD            : out std_logic
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture BDU_ARCH of BDU_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================
constant DIST_MAX       : integer := 8; --DIST_MAX == BOX_SIZE, reference value

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- data signals
signal sig_b_0                   : std_logic;
signal sig_b_1                   : std_logic;
signal sig_b_2                   : std_logic;
signal sig_b_3                   : std_logic;
-- control signals
signal sig_vld_cmp0              : std_logic;
signal sig_vld_cmp1              : std_logic;
signal sig_vld_cmp2              : std_logic;
signal sig_vld_cmp3              : std_logic;
signal sig_vld                   : std_logic;

begin
   -- reset and write to output
   reset : process(RST, sig_b_0 ,sig_b_1, sig_b_2, sig_b_3, sig_vld_cmp0, sig_vld_cmp1, sig_vld_cmp2, sig_vld_cmp3, CLK, ACT)
   begin
      if (RST = '1') then
         OUTPUT<= (others => '1'); 
         sig_vld <= '0';
      elsif (ACT = '1') then
         if (CLK = '1' and CLK'event) then  
            OUTPUT(0) <= sig_b_0; -- create one vector from four std_logic     
            OUTPUT(1) <= sig_b_1; 
            OUTPUT(2) <= sig_b_2; 
            OUTPUT(3) <= sig_b_3;    
            OUTPUT(DATA_WIDTH-1 downto 4) <= (others => '0');            
            sig_vld <= sig_vld_cmp0 and sig_vld_cmp1 and sig_vld_cmp2 and sig_vld_cmp3;
         end if;
      else
         sig_vld <= '0';
      end if;
   end process;
   
   VLD <= sig_vld;
   
   -- comparators (compare barrier distance and reference value)
   cmp0 : process(ACT, S_0)
   begin
      -- detect barrier in front of S_0
      if (ACT = '1') then
         if (to_integer(unsigned(S_0)) < DIST_MAX) then
            sig_b_0 <= '1';
         else
            sig_b_0 <= '0';
         end if;
         sig_vld_cmp0 <= '1';
      else
         sig_b_0 <= '0';
         sig_vld_cmp0 <= '0';
      end if;
   end process;
   
   cmp1 : process(ACT, S_1)
   begin
      -- detect barrier in front of S_1
      if (ACT = '1') then
         if (to_integer(unsigned(S_1)) < DIST_MAX) then
            sig_b_1 <= '1';
         else
            sig_b_1 <= '0';
         end if;
         sig_vld_cmp1 <= '1';
       else
         sig_b_1 <= '0';
         sig_vld_cmp1 <= '0';
      end if;
   end process;
   
   cmp2 : process(ACT, S_2)
   begin
      -- detect barrier in front of S_2
      if (ACT = '1') then
         if (to_integer(unsigned(S_2)) < DIST_MAX) then
            sig_b_2 <= '1';
         else
            sig_b_2 <= '0';
         end if;
         sig_vld_cmp2 <= '1';
       else
         sig_b_2 <= '0';
         sig_vld_cmp2 <= '0';
      end if;
   end process;
   
   cmp3 : process(ACT, S_3)
   begin
      -- detect barrier in front of S_2
      if (ACT = '1') then
         if (to_integer(unsigned(S_3)) < DIST_MAX) then
            sig_b_3 <= '1';
         else
            sig_b_3 <= '0';
         end if;
         sig_vld_cmp3 <= '1';
      else
         sig_b_3 <= '0';
         sig_vld_cmp3 <= '0';
      end if;
   end process;
   
end architecture;
