-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    bram.vhd
-- Description:  Using Block RAM as Memory for Map
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         11.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

Library UNISIM;
use UNISIM.vcomponents.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity BRAM_ENT is
   generic
   (
      DATA_WIDTH  : integer := 4; 
      ADDR_WIDTH  : integer := 13         
   );
   
   port
   (  
      DOA            : out  std_logic_vector(DATA_WIDTH-1 downto 0);  -- 32-bit A port data output
      DOB            : out  std_logic_vector(DATA_WIDTH-1 downto 0); -- 32-bit B port data output
      ADDRA          : in  std_logic_vector(ADDR_WIDTH-1 downto 0);  -- 16-bit A port address input
      ADDRB          : in  std_logic_vector(ADDR_WIDTH-1 downto 0); -- 16-bit B port address input
      CLKA           : in  std_logic; -- 1-bit A port clock input
      CLKB           : in  std_logic; -- 1 bit B port clock input
      DIA            : in  std_logic_vector(DATA_WIDTH-1 downto 0); -- 32-bit A port data input
      DIB            : in  std_logic_vector(DATA_WIDTH-1 downto 0); -- 32-bit B port data input
      ENA            : in  std_logic; -- 1-bit A port enable input
      ENB            : in  std_logic; -- 1-bit B port enable input
      SSRA           : in  std_logic; -- 1-bit A port set/reset input
      SSRB           : in  std_logic; -- 1-bit B port set/reset input
      WEA            : in  std_logic; -- 4-bit A port write enable input
      WEB            : in  std_logic -- 4-bit B port write enable input
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture BRAM_ARCH of BRAM_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- data signals
signal sig_doa          : std_logic_vector(31 downto 0);
signal sig_dob          : std_logic_vector(31 downto 0);
signal sig_dia          : std_logic_vector(31 downto 0);
signal sig_dib          : std_logic_vector(31 downto 0);
signal sig_addra        : std_logic_vector(15 downto 0);
signal sig_addrb        : std_logic_vector(15 downto 0);

-- control signals
signal sig_wea          : std_logic_vector(3 downto 0);    
signal sig_web          : std_logic_vector(3 downto 0);

begin

   -- instantiate block RAM library primitive RAMB36
   RAMB36_inst : RAMB36
   generic map (
      DOA_REG => 0, -- Optional output register on A port (0 or 1)
      DOB_REG => 0, -- Optional output register on B port (0 or 1)
      INIT_A => X"000000000", -- Initial values on A output port
      INIT_B => X"000000000", -- Initial values on B output port
      RAM_EXTENSION_A => "NONE", -- "UPPER", "LOWER" or "NONE" when cascaded
      RAM_EXTENSION_B => "NONE", -- "UPPER", "LOWER" or "NONE" when cascaded
      READ_WIDTH_A => 4, -- Valid values are 1, 2, 4, 9, 18, or 36
      READ_WIDTH_B => 4, -- Valid values are 1, 2, 4, 9, 18, or 36
      SIM_COLLISION_CHECK => "ALL", -- Collision check enable "ALL", "WARNING_ONLY",
                                    -- "GENERATE_X_ONLY" or "NONE"
      SIM_MODE => "SAFE", -- Simulation: "SAFE" vs "FAST", see "Synthesis and Simulation
                           -- Design Guide" for details
      SRVAL_A => X"000000000", -- Set/Reset value for A port output
      SRVAL_B => X"000000000", -- Set/Reset value for B port output
      WRITE_MODE_A => "WRITE_FIRST", -- "WRITE_FIRST", "READ_FIRST" or "NO_CHANGE"
      WRITE_MODE_B => "WRITE_FIRST", -- "WRITE_FIRST", "READ_FIRST" or "NO_CHANGE"
      WRITE_WIDTH_A => 4, -- Valid values are 1, 2, 3, 4, 9, 18, 36
      WRITE_WIDTH_B => 4 -- Valid values are 1, 2, 3, 4, 9, 18, 36
   )
   port map (
      DOA => sig_doa, -- 32-bit A port data output
      DOB => sig_dob, -- 32-bit B port data output
      DOPA => open, -- 4-bit A port parity data output
      DOPB => open, -- 4-bit B port parity data output
      ADDRA => sig_addra, -- 16-bit A port address input
      ADDRB => sig_addrb, -- 16-bit B port address input
      CLKA => CLKA, -- 1-bit A port clock input
      CLKB => CLKB, -- 1 bit B port clock input
      DIA => sig_dia, -- 32-bit A port data input
      DIB => sig_dib, -- 32-bit B port data input
      DIPA => "0000", -- 4-bit A port parity data input
      DIPB => "0000", -- 4-bit B port parity data input
      ENA => ENA, -- 1-bit A port enable input
      ENB => ENB, -- 1-bit B port enable input
      REGCEA => '0', -- 1-bit A port register enable input
      REGCEB => '0', -- 1-bit B port register enable input
      SSRA => SSRA, -- 1-bit A port set/reset input
      SSRB => SSRB, -- 1-bit B port set/reset input
      WEA => sig_wea, -- 4-bit A port write enable input
      WEB => sig_web, -- 4-bit B port write enable input
      CASCADEINLATA => '0', -- 1-bit cascade A latch input
      CASCADEINLATB => '0', -- 1-bit cascade B latch input
      CASCADEINREGA => '0', -- 1-bit cascade A register input
      CASCADEINREGB => '0', -- 1-bit cascade B register input
      CASCADEOUTLATA => open, -- 1-bit cascade A latch output
      CASCADEOUTLATB => open, -- 1-bit cascade B latch output
      CASCADEOUTREGA => open, -- 1-bit cascade A register output
      CASCADEOUTREGB => open -- 1-bit cascade B register output
   );
   -- End of RAMB36_inst instantiation 

   -- routing signals
   DOA <= sig_doa(DATA_WIDTH-1 downto 0);  
   DOB <= sig_dob(DATA_WIDTH-1 downto 0);    
                    
   sig_dia(3 downto 0) <= DIA;        
   sig_dib(3 downto 0) <= DIB;          
   sig_addra(14 downto 2) <= ADDRA;     
   sig_addrb(14 downto 2) <= ADDRB;       

   sig_wea <= WEA & WEA & WEA & WEA;        
   sig_web <= WEB & WEB & WEB & WEB;         
   
end architecture;