-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    muu.vhd
-- Description:  Memory Update Unit
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.1.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity MUU_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16      
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic; 
      -- ----------------- INPUT/OUTPUT INTERFACE ---------------------------
      ACT            : in  std_logic;
      X              : in  std_logic_vector(DATA_WIDTH-1 downto 0);  
      Y              : in  std_logic_vector(DATA_WIDTH-1 downto 0); 
      VALU           : in  std_logic;    
      VLD            : out std_logic;       
      -- ----------------- MEMORY INTERFACE --------------------------------- 
      ADDR_X         : out std_logic_vector(DATA_WIDTH-1 downto 0);
      ADDR_Y         : out std_logic_vector(DATA_WIDTH-1 downto 0);
      RD_WR          : out std_logic;
      DATA_OUT       : out std_logic_vector(3 downto 0);
      DATA_IN        : in std_logic_vector(3 downto 0);
      MEM_ACT        : out std_logic;
      MEM_VLD        : in std_logic  
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture MUU_ARCH of MUU_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state is (init_state, mem_read_state, mem_write_state, idle); --stavy FSM

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- data signals
signal sig_read_value         : std_logic_vector(3 downto 0);
signal sig_data_out           : std_logic_vector(3 downto 0);
-- control signals
signal sig_vld                : std_logic;
signal sig_read               : std_logic;
--signal sig_write              : std_logic;
signal sig_rd_wr              : std_logic;
signal sig_mem_act            : std_logic;
signal state_reg, state_next  : state; --aktualni a budouci stav FSM

begin
   ADDR_X <= X;
   ADDR_Y <= Y;
   RD_WR <= sig_rd_wr;
   MEM_ACT <= sig_mem_act;
   DATA_OUT <= sig_data_out; 
   
   -- read value frome memory
   mem_read : process(sig_read, DATA_IN, CLK)
   begin
      if( rising_edge(CLK) ) then   
         if (sig_read = '1' and MEM_VLD = '1') then
            sig_read_value <= DATA_IN(3 downto 0);
         end if;
      end if;
   end process;

   -- get bits to output
   sig_data_out <= (sig_read_value(3), sig_read_value(2), '1', VALU);  

-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic : process (state_reg, ACT, MEM_VLD,sig_read_value)
   begin
     state_next <= state_reg;  
     
     case state_reg is
        when init_state =>
          if (ACT = '1') then 
             state_next <= mem_read_state;
             sig_vld <= '0';
             sig_mem_act <= '0';
          else 
             state_next <= init_state;
             sig_vld <= '0';
             sig_mem_act <= '0';
          end if;                
        
        when mem_read_state => 
          if (MEM_VLD = '1') then 
             state_next <= idle;
             sig_vld <= '0';
             sig_mem_act <= '1';
          else 
             state_next <= mem_read_state; 
             sig_vld <= '0';
             sig_mem_act <= '1';
          end if;
          
        when idle => 
          if (sig_read_value(1) = '1' and ACT = '1') then 
            state_next <= idle;
            sig_vld <= '1';
            sig_mem_act <= '0';
          elsif (sig_read_value(1) = '1' and ACT = '0') then 
            state_next <= init_state;
            sig_vld <= '0';
            sig_mem_act <= '0';
          else
            state_next <= mem_write_state;
            sig_vld <= '0';
            sig_mem_act <= '0';
          end if;
        
        when mem_write_state =>
          if (ACT = '0') then 
             state_next <= init_state;
             sig_vld <= '0';
             sig_mem_act <= '0';
          elsif (MEM_VLD = '1') then 
             state_next <= mem_write_state;
             sig_vld <= '1';
             sig_mem_act <= '1';
          else 
             state_next <= mem_write_state; 
             sig_vld <= '0';
             sig_mem_act <= '1';
          end if;
      end case;      
   end process;  

   VLD <= sig_vld;
   
-- Moore output logic
   moore_output : process (state_reg)
   begin
      -- default values
      sig_read       <= '0';
            
      case state_reg is
         when init_state         => sig_read       <= '0';
                                    sig_rd_wr      <= '0';
                                    
         when mem_read_state     => sig_read       <= '1';
                                    sig_rd_wr      <= '0';
                                    
         when idle               => sig_read       <= '0';
                                    sig_rd_wr      <= '0';
                                    
         when mem_write_state    => sig_read       <= '0';
                                    sig_rd_wr      <= '1';
      end case;   
   end process moore_output;
   
end architecture;
