-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    peu_fsm.vhd
-- Description:  Controll peu working and getting data from bus
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.1.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
--use ieee.numeric_std.all;
--use ieee.std_logic_unsigned.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity PEU_FSM_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16      
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic; 
      
      -- ----------------- INPUT/OUTPUT DATA --------------------- 
      A              : in std_logic_vector(DATA_WIDTH-1 downto 0);
      B              : in std_logic_vector(DATA_WIDTH-1 downto 0);
      C              : in std_logic_vector(DATA_WIDTH-1 downto 0);

      NOT_CYC_WRITE  : in std_logic;

      X              : out std_logic_vector(DATA_WIDTH-1 downto 0);
      Y              : out std_logic_vector(DATA_WIDTH-1 downto 0);
      
      -- ----------------- PEU DATA ---------------------------
      X_PEU          : in std_logic_vector(DATA_WIDTH-1 downto 0);
      Y_PEU          : in std_logic_vector(DATA_WIDTH-1 downto 0);
      
      A_PEU          : out std_logic_vector(DATA_WIDTH-1 downto 0);
      B_PEU          : out std_logic_vector(DATA_WIDTH-1 downto 0);
      C_PEU          : out std_logic_vector(DATA_WIDTH-1 downto 0);
      
      ACT_PEU        : out  std_logic;
      VLD_PEU        : in  std_logic
 
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture PEU_FSM_ARCH of PEU_FSM_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state_type is (init_state, get_value_state, evaluate_state); --stavy FSM

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- FSM signals 
signal state_reg, state_next: state_type;

-- data signals
signal reg_A                  : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_B                  : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_C                  : std_logic_vector(DATA_WIDTH-1 downto 0);

-- control signals
signal sig_we_abc             : std_logic;
signal sig_act                : std_logic;
signal sig_vld                : std_logic;

signal sig_change_val         : std_logic;

begin
   -- detect event on input data => enable evaluating
   change_register : process (CLK, reg_A, reg_B, reg_C, A, B, C, sig_vld, sig_change_val)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            sig_change_val <= '0';
         elsif (reg_A /= A or reg_B /= B or reg_C /= C) then
            sig_change_val <= '1';
         elsif (sig_vld = '1') then
            sig_change_val <= '0';
         end if;
      end if;
   end process;
   
   
   -- write to input registers A,B,C
   input_registers : process (sig_we_abc, CLK, RST, A, B, C)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            reg_A <= (others => '0');
            reg_B <= (others => '0');
            reg_C <= (others => '0');
         elsif (sig_we_abc = '1') then
            reg_A <= A;
            reg_B <= B;
            reg_C <= C;
         end if;
      end if;
   end process;
   
   X        <= X_PEU;
   Y        <= Y_PEU;
    
   A_PEU    <= reg_A;
   B_PEU    <= reg_B;
   C_PEU    <= reg_C;
     
   ACT_PEU  <= sig_act;
   sig_vld  <= VLD_PEU;
   
-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic : process (state_reg, sig_vld, NOT_CYC_WRITE, sig_change_val)
   begin
     state_next <= state_reg;  
     
     case state_reg is
        when init_state =>
          if (NOT_CYC_WRITE = '1' and sig_change_val = '1') then 
             state_next <= get_value_state;
             sig_we_abc <= '1';
             sig_act <= '0';
          else 
             state_next <= init_state;
             sig_we_abc <= '0';
             sig_act <= '0';
          end if;                
        
        when get_value_state => 
          state_next <= evaluate_state;
          sig_we_abc <= '0';
          sig_act <= '1';
          
        when evaluate_state => 
          if (sig_vld = '1') then 
             state_next <= init_state;
             sig_we_abc <= '0';
             sig_act <= '1';
          else 
             state_next <= evaluate_state;
             sig_we_abc <= '0';
             sig_act <= '1';
          end if;
      end case;      
   end process;  
end architecture;
