-----------------------------------------------------------------------------
-- Project Name: Robot Unit
-- File Name:    wb_rd_wr_slave.vhd
-- Description:  Read/write data from/to master
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         25.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity WB_BDU_SLAVE_ENT is
   generic
   (
      DATA_WIDTH     : integer := 16;  
      ADDR_WIDTH     : integer := 4;
      OFFSET_WIDTH   : integer := 4       
   );
   
   port
   (
      CLK            : in std_logic;
      RST            : in std_logic;

      -- ----------------- WISHBONE SLAVE interface -------------------------
      ACK_O          : out std_logic;
      DAT_I          : in std_logic_vector( DATA_WIDTH-1 downto 0 );
      DAT_O          : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      STB_I          : in std_logic;
      ADR_I          : in std_logic_vector( 3 downto 0 );
      CYC_I          : in std_logic;
      WE_I           : in std_logic;
     
      -- ----------------- WRITE SLAVE OUTPUT -----------------------------------      
      NOT_CYC_WRITE  : out std_logic;
      S_0            : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      S_1            : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      S_2            : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      S_3            : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      
      NOT_CYC_READ   : out std_logic;
      BAR            : in std_logic_vector( DATA_WIDTH-1 downto 0 )
   );
   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture WB_BDU_SLAVE_ARCH of WB_BDU_SLAVE_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state_type is (first_edge, second_edge); 

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================
constant const_addr_S_0       : std_logic_vector(3 downto 0):= "0001"; --adress of register S_0
constant const_addr_S_1       : std_logic_vector(3 downto 0):= "0010"; --adress of register S_1
constant const_addr_S_2       : std_logic_vector(3 downto 0):= "0011"; --adress of register S_2
constant const_addr_S_3       : std_logic_vector(3 downto 0):= "0100"; --adress of register S_3

constant const_addr_BAR       : std_logic_vector(3 downto 0):= "0101"; --adress of register BAR

constant const_n_w            : std_logic_vector( 3 downto 0 ) := "0100";--number of writing vaues
constant const_n_r            : std_logic_vector( 3 downto 0 ) := "0001";--number of reading values
-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- FSM signals 
signal state_reg, state_next: state_type;

-- signals
signal sig_we              : std_logic; -- enable write to input registers
signal sig_n_w             : std_logic_vector( 3 downto 0 ); -- number if writing values
signal sig_go_w            : std_logic; -- signal to increment counter of writed values
signal sig_not_cyc_write   : std_logic; -- signal indicate valit values in output registers
signal sig_not_cyc_write_d : std_logic; -- signal indicate valit values in output registers (delayed) 

signal sig_not_cyc_read    : std_logic; -- signal enabled writing to output registers 
signal sig_n_r             : std_logic_vector( 3 downto 0 ); -- number of reading values
signal sig_go_r            : std_logic; -- signal to increment counter of reading values

-- writing registers
signal reg_s_0             : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_s_1             : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_s_2             : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_s_3             : std_logic_vector(DATA_WIDTH-1 downto 0);
-- reading registers
signal reg_bar             : std_logic_vector(DATA_WIDTH-1 downto 0);


begin
-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= first_edge;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic: process( state_reg, STB_I, CYC_I, WE_I )
   begin
      case state_reg is
         when first_edge =>
            if (STB_I = '1' and CYC_I = '1') then
               ACK_O <= '0';
               sig_we <= '0';
               sig_go_w <= '0';
               sig_go_r <= '0';
               state_next <= second_edge;
            else
               ACK_O <= '0';
               sig_we <= '0';
               sig_go_w <= '0';
               sig_go_r <= '0';
               state_next <= first_edge;
            end if;
         when second_edge =>
            if (WE_I = '1') then
               sig_we <= '1';
               sig_go_w <= '1';
               sig_go_r <= '0';
            else
               sig_we <= '0';
               sig_go_w <= '0';
               sig_go_r <= '1';
            end if;
            ACK_O <= '1';
            state_next <= first_edge;
         when others =>
            ACK_O <= '0';
            sig_we <= '0';
            sig_go_w <= '0';
            sig_go_r <= '0';
            state_next <= first_edge;
      end case;
   end process;  
 
   -- count nubers of send values
   counter_r : process (sig_go_r, RST, CLK)
   variable COUNT: std_logic_vector(3 downto 0):= "0000";
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            COUNT := "0000";
         elsif (sig_go_r='1') then
            if COUNT = const_n_r then
               COUNT := "0001";
            else
               COUNT := COUNT + 1; 
            end if;
         end if; 
      end if;
      sig_n_r <= COUNT;
   end process;
   
   -- enable write to input registers
   sig_not_cyc_read <= '1' when (((sig_n_r = const_n_r) or (sig_n_r = "0000")) and (sig_go_r = '0')) else '0';
   
   NOT_CYC_READ <= sig_not_cyc_read;
   
   -- write to input registers (output registers from bus wiev)
   input_registers : process (sig_not_cyc_read, CLK, RST)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            reg_bar <= (others => '0');
         elsif (sig_not_cyc_read = '1') then
            reg_bar <= BAR;
         end if;
      end if;
   end process;
   
   -- multiplexor to set output data
   mux : process(ADR_I, reg_bar)
   begin
      case ADR_I is
         when const_addr_BAR  => DAT_O <= reg_bar;
         when others          => DAT_O <= (others => '0'); 
      end case;  
   end process;
   
   -- count nubers of receive values   
   counter_w : process (sig_go_w, RST, CLK)
   variable COUNT: std_logic_vector(3 downto 0):= "0000";
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            COUNT := "0000";
         elsif (sig_go_w='1') then
            if COUNT = const_n_w then
               COUNT := "0001";
            else
               COUNT := COUNT + 1; 
            end if;
         end if; 
      end if;
      sig_n_w <= COUNT;--conv_std_logic_vector(COUNT_INT, 4);
   end process;
   
   -- output registers to store receive values
   write_registers : process (sig_we, CLK, RST)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            reg_s_0 <= (others => '0');
            reg_s_1 <= (others => '0');
            reg_s_2 <= (others => '0');
            reg_s_3 <= (others => '0');
         elsif (sig_we = '1') then
            case ADR_I is
               when const_addr_S_0 => reg_s_0 <= DAT_I;
               when const_addr_S_1 => reg_s_1 <= DAT_I;
               when const_addr_S_2 => reg_s_2 <= DAT_I;
               when const_addr_S_3 => reg_s_3 <= DAT_I;
               when others   => null;
            end case;
         end if;
      end if;
   end process;
		
	S_0 <= reg_s_0;
   S_1 <= reg_s_1;
   S_2 <= reg_s_2;
   S_3 <= reg_s_3;
   
   -- indicate, that all output registers are actuall
   sig_not_cyc_write <= '1' when (sig_n_w = const_n_w) else '0';
   
   -- generate delay to align time with registers
   registr_eval_en : process (sig_not_cyc_write, CLK)
	begin
      if( rising_edge( CLK ) ) then
			if (RST = '1') then
            sig_not_cyc_write_d <= '0';
         else
				sig_not_cyc_write_d <= sig_not_cyc_write;
			end if;
		end if;
	end process;
   
   NOT_CYC_WRITE <= sig_not_cyc_write and sig_not_cyc_write_d;
   
end architecture;
