-----------------------------------------------------------------------------
-- Project Name: Robot Unit
-- File Name:    wb_write_master.vhd
-- Description:  Write data to slave's registers specified by constant adress
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         25.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity WB_SIF_WRITE_MASTER_ENT is
   generic
   (
      DATA_WIDTH     : integer := 16;  
      ADDR_WIDTH     : integer := 8;
      OFFSET_WIDTH   : integer := 4       
   );
   
   port
   (
      CLK            : in std_logic;
      RST            : in std_logic;
      ACT            : in std_logic;
      VLD            : out std_logic;

      -- ----------------- WISHBONE MASTER interface -------------------------
      ACK_I          : in std_logic;
      DAT_I          : in std_logic_vector( DATA_WIDTH-1 downto 0 );
      DAT_O          : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      STB_O          : out std_logic;
      ADR_O          : out std_logic_vector( ADDR_WIDTH-1 downto 0 );
      CYC_O          : out std_logic;
      WE_O           : out std_logic; 
     
      -- ----------------- DATA and ADRESS -----------------------------------      
      A         : in std_logic_vector(DATA_WIDTH-1 downto 0);-- := "1010101010101010";--(others => '0');
      B         : in std_logic_vector(DATA_WIDTH-1 downto 0);-- := "1111000011110000";--(others => '0');
      C         : in std_logic_vector(DATA_WIDTH-1 downto 0);-- := "0000111100001111"--(others => '0');
      S_0       : in std_logic_vector(DATA_WIDTH-1 downto 0);-- := "1111000011110000";--(others => '0');
      S_1       : in std_logic_vector(DATA_WIDTH-1 downto 0);-- := "1111000011110000";--(others => '0');
      S_2       : in std_logic_vector(DATA_WIDTH-1 downto 0);-- := "1111000011110000";--(others => '0');
      S_3       : in std_logic_vector(DATA_WIDTH-1 downto 0)-- := "1111000011110000";--(others => '0');
   );
   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture WB_SIF_WRITE_MASTER_ARCH of WB_SIF_WRITE_MASTER_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state_type is (init_state,first_edge_state, second_edge_state, wait_state, handshake_state);

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================
constant const_in_num      : std_logic_vector(3 downto 0) := "0111"; -- number of writing values
constant const_addr_A      : std_logic_vector(ADDR_WIDTH-1 downto 0):= "00010001"; --adress of register A
constant const_addr_B      : std_logic_vector(ADDR_WIDTH-1 downto 0):= "00010010"; --adress of register B
constant const_addr_C      : std_logic_vector(ADDR_WIDTH-1 downto 0):= "00010100"; --adress of register C
constant const_addr_S_0    : std_logic_vector(ADDR_WIDTH-1 downto 0):= "00100001"; --adress of register S_0
constant const_addr_S_1    : std_logic_vector(ADDR_WIDTH-1 downto 0):= "00100010"; --adress of register S_1
constant const_addr_S_2    : std_logic_vector(ADDR_WIDTH-1 downto 0):= "00100011"; --adress of register S_2
constant const_addr_S_3    : std_logic_vector(ADDR_WIDTH-1 downto 0):= "00100100"; --adress of register S_3
-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- FSM signals 
signal state_reg, state_next : state_type;

signal sig_data   : std_logic_vector( DATA_WIDTH-1 downto 0 );--data to write
signal sig_addr   : std_logic_vector( ADDR_WIDTH-1 downto 0);--data address

signal n          : std_logic_vector( 3 downto 0 );--number of writed values
signal sig_go     : std_logic; --signal to increment counter of writed values
signal sig_reset  : std_logic; --signal to reset counter of writed values

begin
-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic: process(state_reg, ACT, ACK_I, n) 
   begin
      state_next <= state_reg;
      case state_reg is
         when init_state => 
            if (ACT = '1') then
               sig_go <= '1';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= first_edge_state;
            else
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= init_state;
            end if;
         when first_edge_state =>
            sig_go <= '0';--
            sig_reset <= '0';
            VLD <= '0';
            state_next <= second_edge_state;   
         when second_edge_state =>
            if (ACK_I = '1' and n = const_in_num) then
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= handshake_state;
            elsif (ACK_I = '1') then
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= wait_state;
            else
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= second_edge_state;
            end if;
         when wait_state => 
            if (n = const_in_num) then
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= init_state;
            else
               sig_go <= '1';--
               sig_reset <= '0';
               VLD <= '0';
               state_next <= first_edge_state;
            end if;
         when handshake_state =>
            if (ACT = '1') then
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '1';
               state_next <= handshake_state; 
            else
               sig_go <= '0';
               sig_reset <= '1';
               VLD <= '0';
              state_next <= init_state;
         end if;
      end case;
   end process;  
   
-- Moore output logic
   moore_output_logic: process(state_reg, sig_addr, sig_data)
   begin  
      case state_reg is
         when init_state => 
            STB_O <= '0';
            CYC_O <= '0';
            WE_O <= '0';
         when first_edge_state =>
            STB_O <= '1';
            CYC_O <= '1';
            WE_O <= '1';
         when second_edge_state =>    
            STB_O <= '1';
            CYC_O <= '1';
            WE_O <= '1';  
         when wait_state =>    
            STB_O <= '0';
            CYC_O <= '1';
            WE_O <= '1';     
         when handshake_state =>    
            STB_O <= '0';
            CYC_O <= '0';
            WE_O <= '0';
         when others =>
            CYC_O <= '0';
            STB_O <= '0';
            WE_O <= '0';
      end case;
   end process;
   
   -- counter to count number of writing values
   counter : process (sig_go, sig_reset, CLK, RST)
   variable COUNT_INT: integer;
   begin
      if( rising_edge( CLK ) ) then
         if (sig_reset = '1' or RST = '1') then
            COUNT_INT := 0;
         elsif (sig_go='1') then
            if COUNT_INT = 15 then
               COUNT_INT := 0;
            else
               COUNT_INT := COUNT_INT + 1; 
            end if;
         end if; 
      end if;
      n <= conv_std_logic_vector(COUNT_INT, 4);
   end process;
   
   -- set writing address and data to signals
   mux : process(n, A, B, C, S_0, S_1, S_2, S_3)
   begin
      case n is
         when "0001"   =>  sig_data <= A;
                           sig_addr <= const_addr_A;
         when "0010"   =>  sig_data <= B;
                           sig_addr <= const_addr_B;
         when "0011"   =>  sig_data <= C;
                           sig_addr <= const_addr_C;
         when "0100"   =>  sig_data <= S_0;
                           sig_addr <= const_addr_S_0;
         when "0101"   =>  sig_data <= S_1;
                           sig_addr <= const_addr_S_1;
         when "0110"   =>  sig_data <= S_2;
                           sig_addr <= const_addr_S_2;
         when "0111"   =>  sig_data <= S_3;
                           sig_addr <= const_addr_S_3;
         when others =>    sig_data <= (others => '0');
                           sig_addr <= (others => '0');   
      end case;  
   end process;
   
   -- set writing address and data to outputs
   ADR_O <= sig_addr;
   DAT_O <= sig_data;
   
end architecture;
